@@ -18,7 +18,7 @@ sub new
     my( $proto, $expression, %kwargs ) = @_;
     my $result = undef;
 
-    $expression =~ s/(^\s+|\s+$)//xgsi;
+    $expression =~ s/^\s+|\s+$//xgsi;
     
     if( 
         not $kwargs{'replacement'}          # cache only top-level expressions
@@ -59,12 +59,22 @@ sub _parse_brackets
             $self->backup_expression($1)
         /xge ){};
     
-    die 'Unpaired brackets in: '.$self->{'expression'}
+    die $self->get_parse_error('unpaired brackets in expression')
         if $expression =~ /[()]/;
     
     return $expression;
 }
 
+sub get_parse_error
+{
+    my ($self, $message, @additional) = @_;
+    
+    return $self->SUPER::get_parse_error(
+        $message
+        , @additional
+        , '            Expression: '.$self->{'expression'}
+    );
+}
 
 sub _parse_expression
 {
@@ -75,24 +85,21 @@ sub _parse_expression
     for( my $level = $self->{'level'}; $level < scalar @DTL::Fast::OPS_RE; $level++ )
     {
         my $operators = $DTL::Fast::OPS_RE[$level];
-
         my @result = ();
         my @source = split /
                 (?:^|\s+)
                     ($operators)
                 (?:$|\s+)
-            /x, $expression;
+            /six, $expression;
 
         if( scalar @source > 1 ) 
         {
- #           warn "Parsed $expression as ".Dumper(\@source);
-            
             # processing operands
             while( defined ( my $token = shift @source) )
             {
                 next if $token eq ''; 
                 
-                if( $token =~ /^$operators$/x ) # operation
+                if( $token =~ /^$operators$/six ) # operation
                 {
                     push @result, $token;
                 }
@@ -105,17 +112,9 @@ sub _parse_expression
             # processing operators
             while( my $token  = shift @result )
             {
-#                warn "Processing token $token";
                 if( ref $token ) # operand
                 {
-                    if( defined $result )
-                    {
-                        die 'Two operands in a row: '.$self->{'expression'};
-                    }
-                    else
-                    {
-                        $result = $token;
-                    }
+                    $result = $token;
                 }
                 else    # operator
                 {
@@ -136,7 +135,7 @@ sub _parse_expression
                             $DTL::Fast::OPS_HANDLERS{$token} = $DTL::Fast::KNOWN_OPS_PLAIN{$token};
                         }
                         
-                        my $handler = $DTL::Fast::OPS_HANDLERS{$token} || die "There is no processor for $token operator";
+                        my $handler = $DTL::Fast::OPS_HANDLERS{$token} || die $self->get_parse_error("there is no processor for $token operator");
                         
                         if($handler->isa('DTL::Fast::Expression::Operator::Binary'))
                         {
@@ -146,9 +145,10 @@ sub _parse_expression
                             }
                             else
                             {
-                                die sprintf('Binary operator %s has no left argument: %s'
-                                    , $token // 'undef'
-                                    , $self->{'expression'} // 'undef'
+                                die $self->get_parse_error(
+                                    sprintf('binary operator `%s` has no left argument'
+                                        , $token // 'undef'
+                                    )
                                 );
                             }
                         }
@@ -156,9 +156,10 @@ sub _parse_expression
                         {
                             if( defined $result )
                             {
-                                die sprintf('Unary operator %s got left argument: %s'
-                                    , $token // 'undef'
-                                    , $self->{'expression'} // 'undef'
+                                die $self->get_parse_error(
+                                    sprintf('unary operator `%s` got left argument'
+                                        , $token // 'undef'
+                                    )
                                 );
                             }
                             else
@@ -168,18 +169,15 @@ sub _parse_expression
                         }
                         else
                         {
-                            die 'Unknown operator handler: '.$handler;
+                            die $self->get_parse_error('Unknown operator handler: '.$handler);
                         }
                     }
                     else # got operator but there is no more operands
                     {
-                        die sprintf('No right argument for %s (%s, %s, %s, %s): %s'
-                            , $token // 'undef'
-                            , scalar @result
-                            , ref $result[0] // 'undef'
-                            , $result // 'undef'
-                            , $result[0] // 'undef'
-                            , $self->{'expression'} // 'undef'
+                        die $self->get_parse_error(
+                            sprintf('operator `%s` has no right argument'
+                                , $token // 'undef'
+                            )
                         );
                     }
                 }
